@@ -7,7 +7,6 @@ use parent 'Queue::Q::ReliableFIFO';
 use Queue::Q::ReliableFIFO::Item;
 use Queue::Q::ReliableFIFO::Lua;
 use Redis;
-use Scalar::Util qw(blessed);
 use Time::HiRes;
 use Data::Dumper;
 
@@ -198,13 +197,8 @@ sub __requeue_busy  {
             $_->_serialized, 
             $self->requeue_limit,
             $place,
-            # NB: last item to ->call can't be undef,
-            # or you'll get "ERR Protocol error: invalid bulk length"
             $error || '',
-        ) for map(blessed($_) && $_->isa('Queue::Q::ReliableFIFO::Item')
-                    ? $_
-                    : Queue::Q::ReliableFIFO::Item->new(data => $_),
-                  @_);
+        ) for @_;
         1;
     }
     or do {
@@ -247,7 +241,7 @@ sub get_and_flush_failed_items {
         if (keys %options);
     my @failures = 
         grep { $_->time_created < ($now-$min_age) 
-                && $_->fail_count >= $min_fc }
+                || $_->fail_count >= $min_fc }
         $self->raw_items_failed();
     my $conn = $self->redis_conn;
     my $name = $self->_failed_queue;
@@ -329,7 +323,7 @@ sub memory_usage_perc {
 }
 
 my %valid_options       = map { $_ => 1 } (qw(
-    Chunk DieOnError MaxItems ProcessAll Pause));
+    Chunk DieOnError MaxItems ProcessAll Pause ReturnWhenEmpty));
 my %valid_error_actions = map { $_ => 1 } (qw(drop requeue ));
 
 sub consume {
@@ -354,6 +348,7 @@ sub consume {
     my $maxitems    = delete $options->{MaxItems} || -1;
     my $pause       = delete $options->{Pause} || 0;
     my $process_all = delete $options->{ProcessAll} || 0;
+    my $return_when_empty= delete $options->{ReturnWhenEmpty} || 0;
     croak("Option ProcessAll without Chunks does not make sense")
         if $process_all && $chunk <= 1;
     croak("Option Pause does without Chunks does not make sense")
@@ -371,7 +366,10 @@ sub consume {
         my $die_afterwards = 0;
         while(!$stop) {
             my $item = eval { $self->claim_item(); };
-            last if (!$item);   #if we don't get an item in claim_wait_timeout, exit
+            if (!$item) {
+                last if $return_when_empty;
+                next;    # nothing claimed this time, try again
+            }
             my $ok = eval { $callback->($item->data); 1; };
             if (!$ok) {
                 my $error = _clean_error($@);
@@ -419,7 +417,10 @@ sub consume {
                 print "error with claim\n";
             };
             $t0 = Time::HiRes::time() if $pause; # only relevant for pause
-            last if (@items == 0);
+            if (@items == 0) {
+                last if $return_when_empty;
+                next;    # nothing claimed this time, try again
+            }
             my @done;
             if ($process_all) {
                 # process all items in one call (option ProcessAll)
@@ -735,11 +736,22 @@ the length of the queue after the items are added.
 This method is called by the consumer to consume the items of a
 queue. For each item in the queue, the callback function will be
 called. The function will receive that data of the queued item
-as parameter. This method also uses B<claim_wait_timeout>.
+as parameter. While the consume method deals with the queue related
+actions, like claiming, "marking as done" etc, the callback function
+only deals with processing the item.
 
 The $action parameter is applied when the callback function returns
 a "die". Allowed values are:
 
+By default, the consume method will keep on reading the queue forever or
+until the process receives a SIGINT or SIGTERM signal. You can make the
+consume method return ealier by using one of the options MaxItems or
+ReturnWhenEmpty. If you still want to have a "near real time" behavior
+you need to make sure there are always consumers running, which can be
+achived using cron and IPC::ConcurrencyLimit::WithStandby.
+
+This method also uses B<claim_wait_timeout>.
+
 =over
 
 =item * B<requeue>. (C<Default>). I.e. do it again, the item will be put at the
@@ -772,10 +784,17 @@ callback function returns a "die" call. Default "false".
 
 =item * B<MaxItems>
 This can be used to limit the consume method to process only a limited amount
-of items. This be useful in cases of memory leaks and restarting
+of items, which can be useful in cases of memory leaks. When you use
+this option, you will probably need to look into restarting 
 strategies with cron. Of course this comes with delays in handling the
 items.
 
+=item * B<ReturnWhenEmpty>
+Use this when you want to let consume() return when the queue is empty.
+Note that comsume will wait for 
+claim_wait_timeout seconds until it can come to the conclusion
+that the queue is empty.
+
 =item * B<Pause>.
 This can be used to give the queue some time to grow, so that more
 items at the time can be claimed. The value of Pause is in seconds and 
@@ -876,6 +895,8 @@ Only the items that failed at least $n times will be retrieved and removed.
 
 =back
 
+If both options are used, only one of the two needs to be true to retrieve and remove an item.
+
 =head2 my $age = $q->age($queue_name [,$type]);
 
 This methods returns the age (in seconds) of the oldest item in the queue.
